/*
	Code generation for the following grammar segments:
	
		Expression := ConditionalExpression [AssignmentOperator Expression]

		ConditionalExpression		:= ConditionalOrExpression [ "?" Expression ":" ConditionalExpression]
		ConditionalOrExpression		:= ConditionalAndExpression ( "||" ConditionalAndExpression)*
		ConditionalAndExpression	:= InclusiveOrExpression ( "&&" InclusiveOrExpression)*
		InclusiveOrExpression		:= ExclusiveOrExpression ( "|" ExclusiveOrExpression)*
		ExclusiveOrExpression		:= AndExpression ( "^" AndExpression)*
		AndExpression			:= EqualityExpression ( "&" EqualityExpression)*
		EqualityExpression		:= InstanceOfExpression ( ("==" | "!=") InstanceOfExpression)*
		InstanceOfExpression		:= RelationalExpression ( "instanceof" ConditionalAndExpression)*
		RelationalExpression		:= ShiftExpression ( ("<"|">"|"<="|">=") ShiftExpression)*
		ShiftExpression			:= AdditiveExpression ( ("<<"|">>"|">>>") AdditiveExpression)*
		AdditiveExpression		:= MultiplicativeExpression ( ("+"|"-") ConditionalAndExpression)*
		MultiplicativeExpression	:= UnaryExpression ( ("*"|"/"|"%") UnaryExpression)*
		UnaryExpression			:= ("+"|"-") UnaryExpression
						 | PreIncrementExpression
						 | PreDecrementExpression
						 | UnaryExpressionNotPlusMinus
		PreIncrementExpression		:= "++" PrimaryExpression
		PreDecrementExpression		:= "--" PrimaryExpression
		UnaryExpressionNotPlusMinus 	:= ("~"|"!") UnaryExpression
						 | PostfixExpression
						 | CastExpression
		PostfixExpression		:= PrimaryExpression ["++"|"--"]
		CastExpression			:= "(" Type ")" UnaryExpression
						 | "(" Type ")" UnaryExpressionNotPlusMinus
		PrimaryExpression		:= PrimaryPrefix (PrimarySuffix)*
*/
package salsac.definitions;

import salsac.*;
import salsac.SalsaParserTreeConstants;

public class Expression extends SimpleNode {

	public Expression(int id) 			{ super(id); }
	public Expression(SalsaParser p, int id)	{ super(p, id); }

	public String getJavaCode() {
		//Attempt to see if the user tried to use a token inside an expression,
		//which is illegal SALSA code.  if so generate an error.

		switch (id) {
			case SalsaParserTreeConstants.JJTEXPRESSION:
			case SalsaParserTreeConstants.JJTPRIMARYEXPRESSION:
				String expcode = getChildCode();

				if (id == SalsaParserTreeConstants.JJTEXPRESSION) {
					if (expcode.equals("this")) expcode = "self";
			
				}
				return expcode;

			case SalsaParserTreeConstants.JJTCONDITIONALEXPRESSION:
			case SalsaParserTreeConstants.JJTCONDITIONALOREXPRESSION:
			case SalsaParserTreeConstants.JJTCONDITIONALANDEXPRESSION:
			case SalsaParserTreeConstants.JJTINCLUSIVEOREXPRESSION:
			case SalsaParserTreeConstants.JJTEXCLUSIVEOREXPRESSION:
			case SalsaParserTreeConstants.JJTANDEXPRESSION:
			case SalsaParserTreeConstants.JJTEQUALITYEXPRESSION:
			case SalsaParserTreeConstants.JJTINSTANCEOFEXPRESSION:
			case SalsaParserTreeConstants.JJTRELATIONALEXPRESSION:
			case SalsaParserTreeConstants.JJTSHIFTEXPRESSION:
			case SalsaParserTreeConstants.JJTADDITIVEEXPRESSION:
			case SalsaParserTreeConstants.JJTMULTIPLICATIVEEXPRESSION:
				String code = getChild(0).getJavaCode();
				for (int i = 1; i < children.length; i++) {
					code += " " + getToken(i-1) + " " + getChild(i).getJavaCode();
				}
				return code;

			case SalsaParserTreeConstants.JJTUNARYEXPRESSION:
			case SalsaParserTreeConstants.JJTUNARYEXPRESSIONNOTPLUSMINUS:
				if (tokens != null) return getToken(0).image + getChild(0).getJavaCode();
				else return getChild(0).getJavaCode();

			case SalsaParserTreeConstants.JJTPREINCREMENTEXPRESSION:
			case SalsaParserTreeConstants.JJTPREDECREMENTEXPRESSION:
				return getToken(0).image + getChild(0).getJavaCode();

			case SalsaParserTreeConstants.JJTPOSTFIXEXPRESSION:
				String code1 = getChild(0).getJavaCode();
				if (tokens != null) code1 += getToken(0).image;

				return code1;
			
			case SalsaParserTreeConstants.JJTCASTEXPRESSION:
				return getToken(0).image + getChild(0).getJavaCode() + getToken(1).image + " " + getChild(1).getJavaCode();
		}

		return "";
	}
}
